/*============================================================================
  File:     ShrinkFragmentationDemo.sql

  Summary:  This script shows how data file
			shrink can cause massive index
			fragmentation

  Date:     June 2008

  SQL Server Versions:
		10.0.1600.22 (SS2008 RTM)
		9.00.3068.00 (SS2005 SP2+)
------------------------------------------------------------------------------
  Written By Paul S. Randal, SQLskills.com
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE MASTER;
GO

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;

CREATE DATABASE DBMaint2008;
GO
USE DBMaint2008;
GO

SET NOCOUNT ON;
GO

-- Create the 10MB filler table at the
-- 'front' of the data file
CREATE TABLE filler (
	c1 INT IDENTITY, c2 VARCHAR(8000))
GO

-- Fill up the filler table
DECLARE @a INT;
SELECT @a = 1;
WHILE (@a < 1280) -- insert 10MB
BEGIN
	INSERT INTO filler VALUES (
		REPLICATE ('a', 5000));
	SELECT @a = @a + 1;
END;
GO

-- Create the production table, which
-- will be 'after' the filler table in
-- the data file
CREATE TABLE production (
	c1 INT IDENTITY, c2 VARCHAR (8000));
CREATE CLUSTERED INDEX prod_cl
	ON production (c1);
GO
DECLARE @a INT;
SELECT @a = 1;
WHILE (@a < 1280) -- insert 10MB
BEGIN
	INSERT INTO production VALUES (
		REPLICATE ('a', 5000));
	SELECT @a = @a + 1;
END;
GO

-- check the fragmentation of the production table
SELECT
	avg_fragmentation_in_percent,
	fragment_count
FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	OBJECT_ID ('production'),
	1,
	NULL,
	'LIMITED');
GO

-- drop the filler table, creating 10MB
-- of free space at the 'front' of the
-- data file
DROP TABLE filler;
GO

-- shrink the database
DBCC SHRINKDATABASE (DBMaint2008);
GO

-- check the index fragmentation again
SELECT
	avg_fragmentation_in_percent,
	fragment_count
FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	OBJECT_ID ('production'),
	1,
	NULL,
	'LIMITED');
GO