/*============================================================================
  File:     sp_allocationMetadata.sql

  Summary:  This script cracks the system tables
			to provide top-level metadata about
			a table or index

  Date:     June 2008

  SQL Server Versions:
		10.0.1600.22 (SS2008 RTM)
		9.00.3068.00 (SS2005 SP2+)
------------------------------------------------------------------------------
  Written By Paul S. Randal, SQLskills.com
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE master;
GO

IF OBJECT_ID ('sp_AllocationMetadata') IS NOT NULL
	DROP PROCEDURE sp_AllocationMetadata;
GO

CREATE PROCEDURE sp_AllocationMetadata
(
	@object VARCHAR (128) = NULL
)
AS
SELECT
	OBJECT_NAME (sp.object_id) AS [Object Name],
	sp.index_id AS [Index ID],
	sa.allocation_unit_id AS [Alloc Unit ID],
	sa.type_desc AS [Alloc Unit Type],
	'(' + CONVERT (VARCHAR (6),
		CONVERT (INT,
			SUBSTRING (sa.first_page, 6, 1) +
			SUBSTRING (sa.first_page, 5, 1))) +
	':' + CONVERT (VARCHAR (20),
		CONVERT (INT,
			SUBSTRING (sa.first_page, 4, 1) +
			SUBSTRING (sa.first_page, 3, 1) +
			SUBSTRING (sa.first_page, 2, 1) +
			SUBSTRING (sa.first_page, 1, 1))) +
	')' AS [First Page],
	'(' + CONVERT (VARCHAR (6),
		CONVERT (INT,
			SUBSTRING (sa.root_page, 6, 1) +
			SUBSTRING (sa.root_page, 5, 1))) +
	':' + CONVERT (VARCHAR (20),
		CONVERT (INT,
			SUBSTRING (sa.root_page, 4, 1) +
			SUBSTRING (sa.root_page, 3, 1) +
			SUBSTRING (sa.root_page, 2, 1) +
			SUBSTRING (sa.root_page, 1, 1))) +
	')' AS [Root Page],
	'(' + CONVERT (VARCHAR (6),
		CONVERT (INT,
			SUBSTRING (sa.first_iam_page, 6, 1) +
			SUBSTRING (sa.first_iam_page, 5, 1))) +
	':' + CONVERT (VARCHAR (20),
		CONVERT (INT,
			SUBSTRING (sa.first_iam_page, 4, 1) +
			SUBSTRING (sa.first_iam_page, 3, 1) +
			SUBSTRING (sa.first_iam_page, 2, 1) +
			SUBSTRING (sa.first_iam_page, 1, 1))) +
	')' AS [First IAM Page]	
FROM
	sys.system_internals_allocation_units AS sa,
	sys.partitions AS sp
WHERE
	sa.container_id = sp.partition_id
AND sp.object_id =
	(CASE WHEN (@object IS NULL)
		THEN sp.object_id
		ELSE OBJECT_ID (@object)
	END);
GO

EXEC sys.sp_MS_marksystemobject sp_AllocationMetadata;
GO

--USE AdventureWorks;
--GO
--EXEC sp_AllocationMetadata 'HumanResources.Employee';
--GO
